@@ -26,6 +26,9 @@ KasumiAddWindow::KasumiAddWindow(KasumiDic *aDictionary,
   g_signal_connect(G_OBJECT(window), "delete_event",
                    G_CALLBACK(_call_back_add_window_delete_event), this);
 
+  // tooltips for every widget
+  Tooltips = gtk_tooltips_new();
+
   // creating vbox for text entries, spin button and so on.
   GtkWidget *vbox = gtk_vbox_new(FALSE,0);
   gtk_container_add(GTK_CONTAINER(window),vbox);  
@@ -180,20 +183,22 @@ KasumiAddWindow::KasumiAddWindow(KasumiDic *aDictionary,
                      FALSE,FALSE,0);
 
   VerbOptionRentaiCheck =
-    gtk_check_button_new_with_label(_("TARU connection"));
+    gtk_check_button_new_with_label(_("Can be Meishi when Renyou type"));
   gtk_box_pack_start(GTK_BOX(VerbOptionPane),
                      GTK_WIDGET(VerbOptionRentaiCheck),
                      FALSE,FALSE,0);
   
   // creating box for buttons
   GtkWidget *hbutton_box = gtk_hbutton_box_new();
-  gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbutton_box),1,1);
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbutton_box),GTK_BUTTONBOX_SPREAD);
   gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(hbutton_box),FALSE,FALSE,0);
 
   // creating buttons and configure shortcut key
   GtkWidget *button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("Quit"));
-  gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
+  GtkWidget *button_image = gtk_image_new_from_stock(GTK_STOCK_QUIT,
+                                                     GTK_ICON_SIZE_BUTTON);
+  gtk_container_add(GTK_CONTAINER(button),button_image);
+  gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),TRUE,TRUE,0);
   g_signal_connect(G_OBJECT(button),"clicked",
                    G_CALLBACK(_call_back_add_window_quit),this);
   GtkAccelGroup *accel = gtk_accel_group_new();
@@ -203,10 +208,15 @@ KasumiAddWindow::KasumiAddWindow(KasumiDic *aDictionary,
                              getAccelKey(key),
                              getModifierType(key),
                              GTK_ACCEL_VISIBLE);
+  gtk_tooltips_set_tip(Tooltips, button,
+                       _("Quit this application"),
+                       _("Save dictionary and quit this application."));
 
   button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("Add"));
-  gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
+  button_image = gtk_image_new_from_stock(GTK_STOCK_ADD,
+                                          GTK_ICON_SIZE_BUTTON);
+  gtk_container_add(GTK_CONTAINER(button),button_image);
+  gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),TRUE,TRUE,0);
   g_signal_connect(G_OBJECT(button),"clicked",
                    G_CALLBACK(_call_back_add_window_add),this);
   accel = gtk_accel_group_new();
@@ -216,10 +226,15 @@ KasumiAddWindow::KasumiAddWindow(KasumiDic *aDictionary,
                              getAccelKey(key),
                              getModifierType(key),
                              GTK_ACCEL_VISIBLE);
+  gtk_tooltips_set_tip(Tooltips, button,
+                       _("Add entered word"),
+                       _("If all the necessary items are filled in, add entered word."));
 
   button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("Manage Mode"));
-  gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
+  button_image = gtk_image_new_from_stock(GTK_STOCK_FIND_AND_REPLACE,
+                                          GTK_ICON_SIZE_BUTTON);
+  gtk_container_add(GTK_CONTAINER(button),button_image);
+  gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),TRUE,TRUE,0);
   g_signal_connect(G_OBJECT(button),"clicked",
                    G_CALLBACK(_call_back_manage_mode),this);
   accel = gtk_accel_group_new();
@@ -229,6 +244,10 @@ KasumiAddWindow::KasumiAddWindow(KasumiDic *aDictionary,
                              getAccelKey(key),
                              getModifierType(key),
                              GTK_ACCEL_VISIBLE);
+  gtk_tooltips_set_tip(Tooltips, button,
+                       _("Manage mode"),
+                       _("Make the shift to manage mode to modify and remove registered words."));
+
 
   // get selection at the time of launching
   g_signal_connect(G_OBJECT(SpellingEntry),"selection_received",
@@ -268,7 +287,7 @@ KasumiAddWindow::KasumiAddWindow(KasumiDic *aDictionary,
     gtk_window_move(GTK_WINDOW(window),x,y);
   }
 
-  if(conf->getPropertyValue("DefaultAddingSpelling") == ""){
+  if(conf->getPropertyValue("DefaultAddingSpelling") == "" && conf->getPropertyValueByBool("ImportSelectedText")){
     get_targets(SpellingEntry);
   }
 
@@ -361,27 +380,15 @@ void KasumiAddWindow::ClickedAddButton(GtkWidget *widget){
     }
 
     dictionary->appendWord(word);
-    dictionary->store();
+    //    dictionary->store();
 
     gtk_entry_set_text(GTK_ENTRY(SoundEntry), "");
     gtk_entry_set_text(GTK_ENTRY(SpellingEntry), "");
     const int FREQ_DEFAULT = conf->getPropertyValueByInt("DefaultFrequency");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(FrequencySpin),FREQ_DEFAULT);
     gtk_combo_box_set_active(GTK_COMBO_BOX(WordClassCombo),0);
-  }catch(KasumiInvalidCharacterForSoundException e){
-      string message;
-      message = string(_("Sound must consist of only Hiragana characters. You have entered invalid character: "));
-      message += e.getInvalidCharacter();
-      
-      GtkWidget *dialog = gtk_message_dialog_new
-        (GTK_WINDOW(window),
-         GTK_DIALOG_DESTROY_WITH_PARENT,
-         GTK_MESSAGE_ERROR,
-         GTK_BUTTONS_OK,
-         message.c_str());
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
-      delete (word);
+  }catch(KasumiException e){
+    handleException(e);
   }
 }
 
