@@ -44,46 +44,87 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary,
   gtk_widget_set_size_request(GTK_WIDGET(ScrolledWindow),300,300);
   gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(ScrolledWindow),TRUE,TRUE,0);
 
-
   // creating tree(list) view for words
   GtkWidget *WordListView = gtk_tree_view_new();
   GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
-  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(WordListView),-1,
-                                              _("Spelling"),renderer,
-                                              "text",COL_WORD,
-                                              NULL);
-  gtk_tree_view_column_new();
+  SpellingColumn = gtk_tree_view_column_new_with_attributes(_("Spelling"),
+                                                            renderer,
+                                                            "text",
+                                                            COL_WORD,
+                                                            NULL);
+  gtk_tree_view_column_set_resizable(SpellingColumn, true);
+  gtk_tree_view_insert_column(GTK_TREE_VIEW(WordListView),SpellingColumn,-1);
+
   renderer = gtk_cell_renderer_text_new();
-  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(WordListView),-1,
-                                              _("Sound"),renderer,
-                                              "text",COL_YOMI,
-                                              NULL);
-  gtk_tree_view_column_new();  
+  SoundColumn = gtk_tree_view_column_new_with_attributes(_("Sound"),
+                                                         renderer,
+                                                         "text",
+                                                         COL_YOMI,
+                                                         NULL);
+  gtk_tree_view_column_set_resizable(SoundColumn, true);
+  gtk_tree_view_insert_column(GTK_TREE_VIEW(WordListView),SoundColumn,-1);
+  gtk_tree_view_column_set_clickable(SoundColumn,TRUE);
+  g_signal_connect(G_OBJECT(SoundColumn), "clicked",
+                   G_CALLBACK(_call_back_clicked_column_header), this);
+
   renderer = gtk_cell_renderer_text_new();
+  FreqColumn = gtk_tree_view_column_new_with_attributes(_("Frequency"),
+                                                        renderer,
+                                                        "text",
+                                                        COL_FREQ,
+                                                        NULL);
+  gtk_tree_view_column_set_resizable(FreqColumn, true);
+  gtk_tree_view_insert_column(GTK_TREE_VIEW(WordListView),FreqColumn,-1);
+  gtk_tree_view_column_set_clickable(FreqColumn, TRUE);
+  g_signal_connect(G_OBJECT(FreqColumn), "clicked",
+                   G_CALLBACK(_call_back_clicked_column_header), this);
+  /*
   gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(WordListView),-1,
                                               _("Frequency"),renderer,
                                               "text",COL_FREQ,
                                               NULL);
-  gtk_tree_view_column_new();  
+  */
   renderer = gtk_cell_renderer_text_new();
-  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(WordListView),-1,
-                                              _("Word Class"),renderer,
-                                              "text",COL_PART,
-                                              NULL);
-  WordList = gtk_list_store_new(NUM_COLS,G_TYPE_UINT,G_TYPE_STRING,
-                                G_TYPE_STRING,G_TYPE_UINT,G_TYPE_STRING);
+  WordClassColumn = gtk_tree_view_column_new_with_attributes(_("WordClass"),
+                                                             renderer,
+                                                             "text",
+                                                             COL_PART,
+                                                             NULL);
+  gtk_tree_view_column_set_resizable(WordClassColumn, true);
+  gtk_tree_view_insert_column(GTK_TREE_VIEW(WordListView),WordClassColumn,-1);
+  gtk_tree_view_column_set_clickable(WordClassColumn, TRUE);
+  g_signal_connect(G_OBJECT(WordClassColumn), "clicked",
+                   G_CALLBACK(_call_back_clicked_column_header), this);
+  
+  WordList = gtk_list_store_new(NUM_COLS,G_TYPE_UINT,
+                                G_TYPE_STRING,G_TYPE_STRING,
+                                G_TYPE_UINT,G_TYPE_STRING);
+  SortList = gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(WordList));
+  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(SortList), COL_YOMI,
+                                  sortFuncBySound,(gpointer)dictionary,
+                                  NULL);
+  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(SortList), COL_FREQ,
+                                  sortFuncByFreq,(gpointer)dictionary,
+                                  NULL);
+  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(SortList), COL_PART,
+                                  sortFuncByWordClass,(gpointer)dictionary,
+                                  NULL);
   gtk_tree_view_set_model(GTK_TREE_VIEW(WordListView),
-                          GTK_TREE_MODEL(WordList));
+                          GTK_TREE_MODEL(SortList));
 
-  WordListSelection = gtk_tree_view_get_selection(GTK_TREE_VIEW(WordListView));
-  gtk_tree_selection_set_mode(WordListSelection, GTK_SELECTION_SINGLE);
-  g_signal_connect(G_OBJECT(WordListSelection), "changed",
+  SortListSelection = gtk_tree_view_get_selection(GTK_TREE_VIEW(WordListView));
+  gtk_tree_selection_set_mode(SortListSelection, GTK_SELECTION_SINGLE);
+  g_signal_connect(G_OBJECT(SortListSelection), "changed",
                    G_CALLBACK(_call_back_changed_list_cursor), this);
 
   // destroy model automatically with view
   g_object_unref(GTK_TREE_MODEL(WordList));
 
-  gtk_container_add(GTK_CONTAINER(ScrolledWindow),GTK_WIDGET(WordListView));  
+  gtk_container_add(GTK_CONTAINER(ScrolledWindow),GTK_WIDGET(WordListView));
+
+  // separator between word list box and text entries
+  GtkWidget *vsep = gtk_vseparator_new();
+  gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(vsep),FALSE,FALSE,0);
 
   // creating vbox for text entries
   GtkWidget *entry_vbox = gtk_vbox_new(FALSE,0);
@@ -295,10 +336,15 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary,
     g_signal_connect(G_OBJECT(VerbOptionRentaiCheck),"toggled",
                      G_CALLBACK(_call_back_toggled_check),this);
 
+  GtkWidget *hsep = gtk_hseparator_new();
+  gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(hsep),FALSE,FALSE,0);
+
+  // creating expandable box for search function
+  GtkWidget *search_expander = gtk_expander_new(_("Search"));
+  gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(search_expander),FALSE,FALSE,0);
 
-  // creating box for search function
   GtkWidget *search_hbox = gtk_hbox_new(FALSE,8);
-  gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(search_hbox),FALSE,FALSE,0);
+  gtk_container_add(GTK_CONTAINER(search_expander),search_hbox);
 
   // creating Entry and Button for search function
   SearchEntry = gtk_entry_new();
@@ -327,6 +373,8 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary,
   gtk_box_pack_start(GTK_BOX(search_hbox),SearchBySoundRadio,
                      TRUE,TRUE,0);
 
+  hsep = gtk_hseparator_new();
+  gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(hsep),FALSE,FALSE,0);
 
   /* creating box for buttons */
   GtkWidget *hbutton_box = gtk_hbutton_box_new();
@@ -335,8 +383,7 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary,
 
 
   // creating buttons and shortcut key configuration
-  GtkWidget *button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("Quit"));
+  GtkWidget *button = gtk_button_new_from_stock(GTK_STOCK_QUIT);
   gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
   g_signal_connect(G_OBJECT(button),"clicked",
                    G_CALLBACK(_call_back_quit),this);
@@ -348,8 +395,7 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary,
                              getModifierType(key),
                              GTK_ACCEL_VISIBLE);
 
-  button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("Store"));
+  button = gtk_button_new_from_stock(GTK_STOCK_SAVE);
   gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
   g_signal_connect(G_OBJECT(button),"clicked",
                    G_CALLBACK(_call_back_store),this);
@@ -361,8 +407,7 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary,
                              getModifierType(key),
                              GTK_ACCEL_VISIBLE);
 
-  button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("New Word"));
+  button = gtk_button_new_from_stock(GTK_STOCK_ADD);
   gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
   g_signal_connect(G_OBJECT(button),"clicked",
                    G_CALLBACK(_call_back_add),this);
@@ -374,8 +419,7 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary,
                              getModifierType(key),
                              GTK_ACCEL_VISIBLE);
   
-  button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("Remove"));
+  button = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
   gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
   g_signal_connect(G_OBJECT(button),"clicked",
                    G_CALLBACK(_call_back_remove),this);
@@ -417,7 +461,7 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary,
 }
 
 void KasumiMainWindow::refresh(){
-  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   int i = 0;
 
@@ -428,8 +472,6 @@ void KasumiMainWindow::refresh(){
       KasumiWord *word = dictionary->getWordWithID(i);
 
       if(word != NULL && word->getFrequency() != 0){
-        GtkTreeIter iter;
-      
         gtk_list_store_append(WordList,&iter);
       
         gtk_list_store_set(WordList,&iter,
@@ -440,9 +482,8 @@ void KasumiMainWindow::refresh(){
                            COL_PART,word->getStringOfWordClassByUTF8().c_str(),
                            -1);
       }
-    }catch(KasumiOutOfBoundException e){
-      cout << e.getMessage() << endl;
-      exit(1);
+    }catch(KasumiException e){
+      handleException(e);
     }
   }
 
@@ -462,7 +503,7 @@ void KasumiMainWindow::refresh(){
   }
 
   // select first word
-  gtk_tree_selection_select_iter(WordListSelection,&iter);  
+  gtk_tree_selection_select_iter(SortListSelection,&iter);  
 
 }
 
@@ -504,7 +545,7 @@ void KasumiMainWindow::ClickedStoreButton(){
   try{
     dictionary->store();
     modificationFlag = false;
-  }catch(KasumiDicStoreException e){
+  }catch(KasumiException e){
       cout <<  e.getMessage() << endl;
       exit(1);
   }
@@ -516,22 +557,22 @@ void KasumiMainWindow::ClickedAddButton(){
 }
 
 void KasumiMainWindow::ClickedRemoveButton(){
-  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   int id;
   
-  if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
+  if(gtk_tree_selection_get_selected(SortListSelection, &model, &iter)){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     dictionary->removeWord(id);
   }
 }
 
 void KasumiMainWindow::ChangedListCursor(){
-  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   int id;
   
-  if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
+  if(gtk_tree_selection_get_selected(SortListSelection, &model, &iter)){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     KasumiWord *word = dictionary->getWordWithID(id);
 
@@ -586,30 +627,19 @@ void KasumiMainWindow::ChangedListCursor(){
 }
 
 void KasumiMainWindow::ChangedSoundEntry(){
-  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   int id;
   
-  if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
+  if(gtk_tree_selection_get_selected(SortListSelection, &model, &iter)){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     KasumiWord *word = dictionary->getWordWithID(id);
     try{
       word->setSoundByUTF8(string(gtk_entry_get_text(GTK_ENTRY(SoundEntry))));
       dictionary->modifyWord(id);
       previousSoundEntryText = string(gtk_entry_get_text(GTK_ENTRY(SoundEntry)));
-    }catch(KasumiInvalidCharacterForSoundException e){
-      string message;
-      message = string(_("Sound must consist of only Hiragana characters. You have entered invalid character: "));
-      message += e.getInvalidCharacter();
-      
-      GtkWidget *dialog = gtk_message_dialog_new
-        (GTK_WINDOW(window),
-         GTK_DIALOG_DESTROY_WITH_PARENT,
-         GTK_MESSAGE_ERROR,
-         GTK_BUTTONS_OK,
-         message.c_str());
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
+    }catch(KasumiException e){
+      handleException(e);
 
       // take back invalid entry
       g_signal_handler_block(SoundEntry, HandlerIDOfSoundEntry);
@@ -621,11 +651,11 @@ void KasumiMainWindow::ChangedSoundEntry(){
 }
 
 void KasumiMainWindow::ChangedSpellingEntry(){
-  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   int id;
   
-  if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
+  if(gtk_tree_selection_get_selected(SortListSelection, &model, &iter)){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     KasumiWord *word = dictionary->getWordWithID(id);
     word->setSpellingByUTF8(string(gtk_entry_get_text(
@@ -636,11 +666,11 @@ void KasumiMainWindow::ChangedSpellingEntry(){
 }
 
 void KasumiMainWindow::ChangedFrequencySpin(){
-  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   int id;
   
-  if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
+  if(gtk_tree_selection_get_selected(SortListSelection, &model, &iter)){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     KasumiWord *word = dictionary->getWordWithID(id);
     word->setFrequency(gtk_spin_button_get_value_as_int(
@@ -651,11 +681,11 @@ void KasumiMainWindow::ChangedFrequencySpin(){
 }
 
 void KasumiMainWindow::ChangedWordClassCombo(){
-  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   int id;
   
-  if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
+  if(gtk_tree_selection_get_selected(SortListSelection, &model, &iter)){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     KasumiWord *word = dictionary->getWordWithID(id);
     word->setWordClass(getActiveWordClass());
@@ -668,11 +698,11 @@ void KasumiMainWindow::ChangedWordClassCombo(){
 }
 
 void KasumiMainWindow::ChangedVerbTypeCombo(){
- GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+ GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   int id;
   
-  if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
+  if(gtk_tree_selection_get_selected(SortListSelection, &model, &iter)){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     KasumiWord *word = dictionary->getWordWithID(id);
     word->setVerbType(getActiveVerbType());
@@ -682,7 +712,7 @@ void KasumiMainWindow::ChangedVerbTypeCombo(){
 }
 
 void KasumiMainWindow::ChangedOption(GtkWidget *widget){
-  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   int id;
   string OptionName = string("");
@@ -715,7 +745,7 @@ void KasumiMainWindow::ChangedOption(GtkWidget *widget){
   else
     val = false;
 
-  if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter) &&
+  if(gtk_tree_selection_get_selected(SortListSelection, &model, &iter) &&
      (OptionName != "")){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     KasumiWord *word = dictionary->getWordWithID(id);
@@ -731,7 +761,7 @@ void KasumiMainWindow::SwitchToAddingMode(){
 }
 
 void KasumiMainWindow::FindNext(bool fromCurrent){
-  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeModel *model = GTK_TREE_MODEL(SortList);
   GtkTreeIter iter;
   KasumiWord *word;
   bool fromFirst = false;
@@ -746,7 +776,7 @@ void KasumiMainWindow::FindNext(bool fromCurrent){
     by = SOUND;
   }
 
-  if(!gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
+  if(!gtk_tree_selection_get_selected(SortListSelection, &model, &iter)){
     if(!gtk_tree_model_get_iter_first(model, &iter)){
       // If no words, do nothing.
       return;
@@ -782,7 +812,7 @@ void KasumiMainWindow::FindNext(bool fromCurrent){
 
     if(comparedString.find(searchString,0) == 0){
       // if found, select that word and don't search any more
-      gtk_tree_selection_select_iter(WordListSelection,&iter);
+      gtk_tree_selection_select_iter(SortListSelection,&iter);
       return;
     }
   }while(gtk_tree_model_iter_next(model, &iter));
@@ -814,7 +844,7 @@ void KasumiMainWindow::FindNext(bool fromCurrent){
         }
 
         if(comparedString.find(searchString,0) == 0){
-          gtk_tree_selection_select_iter(WordListSelection,&iter);
+          gtk_tree_selection_select_iter(SortListSelection,&iter);
           return;
         }
       }while(gtk_tree_model_iter_next(model, &iter) &&
@@ -823,7 +853,7 @@ void KasumiMainWindow::FindNext(bool fromCurrent){
               StartIter.user_data3 != iter.user_data3));
     }else{
       gtk_widget_destroy(dialog);
-      //      gtk_tree_selection_unselect_all(WordListSelection);      
+      //      gtk_tree_selection_unselect_all(SortListSelection);      
       return;
     }
   }
@@ -835,7 +865,32 @@ void KasumiMainWindow::FindNext(bool fromCurrent){
                                   _("Cannot find a specific word."));
   gtk_dialog_run(GTK_DIALOG (dialog));
   gtk_widget_destroy(dialog);
-  //  gtk_tree_selection_unselect_all(WordListSelection);
+  //  gtk_tree_selection_unselect_all(SortListSelection);
+}
+
+void KasumiMainWindow::SortBy(GtkTreeViewColumn *column){
+  GtkSortType order;
+
+  order = gtk_tree_view_column_get_sort_order(column);
+  order = (order == GTK_SORT_ASCENDING) ?
+    GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;
+  
+  gtk_tree_view_column_set_sort_indicator(SoundColumn,FALSE);
+  gtk_tree_view_column_set_sort_indicator(FreqColumn,FALSE);
+  gtk_tree_view_column_set_sort_indicator(WordClassColumn,FALSE);
+
+  if(column == SoundColumn){
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(SortList),
+                                         COL_YOMI, order);
+  }else if(column == FreqColumn){
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(SortList),
+                                         COL_FREQ, order);
+  }else if(column == WordClassColumn){
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(SortList),
+                                         COL_PART, order);
+  }
+  gtk_tree_view_column_set_sort_indicator(column,TRUE);
+  gtk_tree_view_column_set_sort_order(column,order);
 }
 
 void KasumiMainWindow::removedWord(int id){
@@ -867,7 +922,10 @@ void KasumiMainWindow::appendedWord(int id){
                      COL_FREQ, word->getFrequency(),
                      COL_PART, word->getStringOfWordClassByUTF8().c_str(),
                      -1);
-  gtk_tree_selection_select_iter(WordListSelection,&iter);
+  GtkTreeIter sort_iter;
+  gtk_tree_model_sort_convert_child_iter_to_iter(GTK_TREE_MODEL_SORT(SortList),
+                                                 &sort_iter,&iter);
+  gtk_tree_selection_select_iter(SortListSelection,&sort_iter);
 
   GtkAdjustment *adjustment =
     gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(ScrolledWindow));
@@ -892,6 +950,7 @@ void KasumiMainWindow::modifiedWord(int id){
   modificationFlag = true;  
 }
 
+// Do not returns iter of SortList but WordList
 GtkTreeIter *KasumiMainWindow::findCorrespondingIter(int id){
   int i;
   GtkTreeModel *model = GTK_TREE_MODEL(WordList);
@@ -1248,6 +1307,13 @@ void _call_back_activate_search_entry(GtkWidget *widget,
   window->FindNext(false);
 }
 
+void _call_back_clicked_column_header(GtkTreeViewColumn *column,
+                                             gpointer data){
+  KasumiMainWindow *window = (KasumiMainWindow *)data;
+  window->SortBy(column);
+}
+
+
 guint getAccelKey(const string &key){
   string::size_type i;
 
@@ -1315,3 +1381,97 @@ GdkModifierType getModifierType(const string &key){
   cout << "Invalid mask option: " << key << endl;
   exit(1);
 }
+
+gint sortFuncByFreq(GtkTreeModel *model,
+                    GtkTreeIter *a,
+                    GtkTreeIter *b,
+                    gpointer user_data){
+  KasumiDic *dictionary = (KasumiDic*)user_data;
+  int id_a, id_b;
+  gtk_tree_model_get(model, a, COL_ID, &id_a, -1);
+  gtk_tree_model_get(model, b, COL_ID, &id_b, -1);
+  KasumiWord *word_a = dictionary->getWordWithID(id_a);
+  KasumiWord *word_b = dictionary->getWordWithID(id_b);
+
+  return word_a->getFrequency() - word_b->getFrequency();
+}
+
+gint sortFuncBySound(GtkTreeModel *model,
+                     GtkTreeIter *iter_a,
+                     GtkTreeIter *iter_b,
+                     gpointer user_data){
+  KasumiDic *dictionary = (KasumiDic*)user_data;
+  int id_a, id_b;
+  gtk_tree_model_get(model, iter_a, COL_ID, &id_a, -1);
+  gtk_tree_model_get(model, iter_b, COL_ID, &id_b, -1);
+  KasumiWord *word_a = dictionary->getWordWithID(id_a);
+  KasumiWord *word_b = dictionary->getWordWithID(id_b);
+  const char *str_a = word_a->getSoundByUTF8().c_str();
+  const char *str_b = word_b->getSoundByUTF8().c_str();
+  int size_a = word_a->getSound().size();
+  int size_b = word_b->getSound().size();
+  int size = (size_a > size_b) ? size_a : size_b;
+  int i,a,b;
+  unsigned char first_a,second_a,third_a;
+  unsigned char first_b,second_b,third_b;
+  
+  // compare Hiragana string encoded in UTF8
+  for(i=0;i<size;i+=3){
+    first_a = static_cast<unsigned char> (str_a[i]);
+    second_a = static_cast<unsigned char> (str_a[i+1]);
+    third_a = static_cast<unsigned char> (str_a[i+2]);
+    first_b = static_cast<unsigned char> (str_b[i]);
+    second_b = static_cast<unsigned char> (str_b[i+1]);
+    third_b = static_cast<unsigned char> (str_b[i+2]);
+
+    // confirm that current characters are Hiragana
+    if(first_a == 0xe3 && first_b == 0xe3){
+      if((second_a == 0x81 && third_a >= 0x81 && third_a <= 0xbf) ||
+         (second_a == 0x82 && third_a >= 0x80 && third_a <= 0x94) ||
+         (second_a == 0x83 && third_a == 0xbc)){
+        a = second_a * 256 + third_a;
+      }else{
+        cout << "invalid character or not hiragana" << endl;
+        cout << str_a << endl;
+        exit(1);
+      }
+
+      if((second_b == 0x81 && third_b >= 0x81 && third_b <= 0xbf) ||
+         (second_b == 0x82 && third_b >= 0x80 && third_b <= 0x94) ||
+         (second_b == 0x83 && third_b == 0xbc)){
+        b = second_b * 256 + third_b;
+      }else{
+        cout << "invalid character or not hiragana" << endl;
+        cout << str_b << endl;
+        exit(1);
+      }
+
+      if(a != b){
+        return a - b;
+      }
+    }else{
+        cout << "invalid character or not hiragana" << endl;
+        cout << str_a << endl;
+        cout << str_b << endl;
+        exit(1);
+    }
+  }
+  
+  // one string is the beginning part of another string
+  // compare string size
+  return size_a - size_b;
+}
+
+gint sortFuncByWordClass(GtkTreeModel *model,
+                         GtkTreeIter *iter_a,
+                         GtkTreeIter *iter_b,
+                         gpointer user_data){
+  KasumiDic *dictionary = (KasumiDic*)user_data;
+  int id_a, id_b;
+  gtk_tree_model_get(model, iter_a, COL_ID, &id_a, -1);
+  gtk_tree_model_get(model, iter_b, COL_ID, &id_b, -1);
+  KasumiWord *word_a = dictionary->getWordWithID(id_a);
+  KasumiWord *word_b = dictionary->getWordWithID(id_b);
+
+  return word_a->getWordClass() - word_b->getWordClass();
+}
