@@ -30,12 +30,14 @@ void showHelp(){
   cout << "  --wordclass val  Set default word class entry (only in Add mode)" << endl;
   cout << "  -x val           Set default horizontal window position" << endl;
   cout << "  -y val           Set default vertical window position" << endl;
+  cout << "  -i --import      Import selected text as a spelling" << endl;
+  cout << "  -I --ignore      Ignore selected text" << endl;
   cout << endl;
 }
 
 void showVersion(){
   cout << "kasumi " << VERSION << ": a personal dictionary manager for Anthy" << endl;
-  cout << "Copyright (C) 2004 Takashi Nakamoto.\n";
+  cout << "Copyright (C) 2004-2005 Takashi Nakamoto.\n";
   cout << "This program comes with NO WARRANTY, to the extent permitted " \
           "by law. \nYou may redistribute it under the terms of the GNU " \
           "General Public License; \nsee the file named COPYING for " \
@@ -70,12 +72,8 @@ int main(int argc, char *argv[])
       new KasumiAddWindow(dic,conf);
       gtk_main();
     }
-  }catch(KasumiConfigurationLoadException e){
-    cout << e.getMessage() << endl;
-    exit(1);
-  }catch(KasumiDicExaminationException e){
-    cout << "line " << e.getLine() << ": " << e.getMessage() << endl;
-    exit(1);
+  }catch(KasumiException e){
+    handleException(e);
   }
 }
 
